package top.daxian1218.dust.edp;

import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.mongodb.core.MongoTemplate;
import top.daxian1218.dust.edp.services.authentication.IUserService;
import top.daxian1218.dust.edp.services.authentication.UserService;
import top.daxian1218.dust.edp.services.storage.MongoFileRepository;
import top.daxian1218.dust.service.config.SecurityConfiguration;
import top.daxian1218.dust.service.security.jwt.TokenProvider;

@Configuration
@AutoConfigureAfter(SecurityConfiguration.class)
@EnableConfigurationProperties(EdpProperties.class)
public class EdpConfiguration {

    @Bean
    @ConditionalOnMissingBean(IUserService.class)
    public IUserService userService(TokenProvider tokenProvider){
        return new UserService(tokenProvider);
    }

    @Bean
    @ConditionalOnProperty("spring.data.mongodb.uri")
    public MongoFileRepository mongoFileRepository(MongoTemplate mongoTemplate) {
        return new MongoFileRepository(mongoTemplate);
    }
}
