package top.daxian1218.dust.edp.component;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import top.daxian1218.dust.commons.data.HDataTable;
import top.daxian1218.dust.commons.exception.DustSystemException;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.DictObjBuilder;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.edp.ui.DictComponent;
import top.daxian1218.dust.edp.ui.MenuStore;
import top.daxian1218.dust.edp.utils.SqlUtil;

import static top.daxian1218.dust.edp.utils.SqlUtil.SCRIPT_SPACE;

@Component
public class DataDictComponent extends DictComponent
{
    @Override
    public Object DoActionAsync(IDataAdapter adapter, String actionCode, MenuStore input) {
        switch (actionCode) {
            case "dictScript":
                String id = input.getState().get("id");
                String dbType =  input.getState().get("dbType");
                if (StringUtils.isEmpty(id) || StringUtils.isEmpty(dbType)) {
                    throw new DustSystemException("字典或者数据库类型参数缺失");
                }
                switch (dbType) {
                    case "oracle":
                        return getDictScriptForOracle(adapter, id);
                }
                return StringUtils.EMPTY;
            default:
                return super.DoActionAsync(adapter, actionCode, input);
        }
    }

    private String getDictScriptForOracle(IDataAdapter adapter, String strId) {
        long id = TypeUtil.toLong(strId);

        DictObj dictData = DictObjBuilder.create(9);
        dictData.search(id, adapter);

        DictObj dictDataItem =  DictObjBuilder.create(10);
        dictDataItem.addCondition("pId", id);
        dictDataItem.search();

        StringBuilder sqlBuilder = new StringBuilder();

        sqlBuilder.append("BEGIN").append(StringUtils.LF);

        // dict_data
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM dict_data WHERE id=").append(id).append(";").append(StringUtils.LF);
        SqlUtil.insertDictData(sqlBuilder, dictData);

        // dict_data_item
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM dict_data_item WHERE p_id=").append(id).append(";").append(StringUtils.LF);
        SqlUtil.insertDictData(sqlBuilder, dictDataItem);

        sqlBuilder.append(SCRIPT_SPACE).append("COMMIT;").append(StringUtils.LF);
        sqlBuilder.append("END;");

        return sqlBuilder.toString();
    }
}