package top.daxian1218.dust.edp.component;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import top.daxian1218.dust.commons.data.HDataTable;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.DictObjRow;
import top.daxian1218.dust.db.dict.Error;
import top.daxian1218.dust.db.dict.RowState;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.edp.AppCodes;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.ui.DictComponent;
import top.daxian1218.dust.edp.ui.MenuStore;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author daxian
 */
@Component
public class RoleComponent extends DictComponent {

    public static final int MAX_ROLE_COUNT = 100;
    public static final String OBJ_ROLE = "SysRole";

    @Override
    public Object DoActionAsync(IDataAdapter adapter, String actionCode, MenuStore input) {
        switch (actionCode) {
            case "copyRight": {
                Long srcRole = TypeUtil.toLong(input.getState().get("srcRole"));
                Long destRole = TypeUtil.toLong(input.getState().get("destRole"));
                if (srcRole == 0 || destRole == 0) {
                    throw new BuzException("传入角色无效");
                }
                copyRight(adapter, srcRole, destRole);
                return new JSONObject();
            }
            case "allRole": {
                return getAllRole(adapter);
            }
            case "checkRoleMenu": {
                long menuId = TypeUtil.toLong(input.getState().get("menuId"));
                boolean checked = TypeUtil.toBoolean(input.getState().get("checked"));
                String roleId = TypeUtil.toString(input.getState().get("roleId"));
                handleMenuActionRight(adapter, roleId ,menuId, checked);
            }
            default: {
                return super.DoActionAsync(adapter, actionCode, input);
            }
        }
    }

    private void handleMenuActionRight(IDataAdapter adapter, String roleId, long menuId, boolean checked) {
        if (StringUtils.isEmpty(roleId)) {
            throw new BuzException("角色不存在");
        }

        if (menuId == 0) {
            DictObj dictUIAction = new DictObj("DictUIAction", AppCodes.DictApp);
            dictUIAction.search(adapter);

            HSqlCommand cmdClear = new HSqlCommand("Delete From sys_role_action where role_id=:roleId");
            cmdClear.setParameter("roleId", TypeUtil.toLong(roleId));
            adapter.update(cmdClear);

            DictObj sysRoleAction = new DictObj("SysRoleAction", AppCodes.SysApp);
            dictUIAction.getRows().forEach(r -> {
                DictObjRow newRow = sysRoleAction.addRow();
                newRow.setValue("roleId", roleId);
                newRow.setValue("actionId", r.getId());
                newRow.setValue("menuId", menuId);
            });
            sysRoleAction.save(adapter);
            adapter.commit();
            return;
        }

        DictObj sysMenu = new DictObj("SysMenu", AppCodes.SysApp);
        sysMenu.search(menuId, adapter);
        if (sysMenu.getRows().size() == 0) {
            throw new BuzException("菜单不存在");
        }

        HSqlCommand cmdClear = new HSqlCommand("DELETE FROM SYS_ROLE_ACTION sra LEFT JOIN SYS_MENU sm ON sm.ID = sra.MENU_ID WHERE sra.ROLE_ID =:roleId AND sm.ORDER_LEVEL LIKE :level");
        cmdClear.setParameter("level", sysMenu.getRow(0).get("orderLevel").getStringValue() + "%");
        cmdClear.setParameter("roleId", roleId);
        adapter.update(cmdClear);

        HSqlCommand cmdAction = new HSqlCommand("SELECT dua.ID, sm.id AS MenuId FROM DICT_UI_ACTION dua LEFT JOIN DICT_UI du ON du.ID = dua.UI_ID LEFT JOIN SYS_MENU sm ON sm.UI_ID =du.ID \n" +
                "WHERE sm.ORDER_LEVEL LIKE :level");
        cmdAction.setParameter("level", sysMenu.getRow(0).get("orderLevel").getStringValue() + "%");
        HDataTable dtAction = adapter.query(cmdAction);

        DictObj sysRoleAction = new DictObj("SysRoleAction", AppCodes.SysApp);
        dtAction.forEach(r -> {
            DictObjRow newRow = sysRoleAction.addRow();
            newRow.setValue("roleId", roleId);
            newRow.setValue("actionId", r.getLong("ID"));
            newRow.setValue("menuId", r.getLong("MenuId"));
        });

        sysRoleAction.save(adapter);
        adapter.commit();
    }

    private JSONObject getAllRole(IDataAdapter adapter) {
        DictObj sysRole = new DictObj("SysRole", AppCodes.SysApp);
        sysRole.getPageInfo().setPageSize(MAX_ROLE_COUNT);
        sysRole.search();
        JSONObject data = new JSONObject();
        data.put("roles", sysRole.toDataJSON());
        return data;
    }

    private void copyRight(IDataAdapter adapter, long srcRole, long destRole) {
        copyRight(adapter, srcRole, destRole, true);
    }

    private List<Long> getRoleMenus(long roleId) {
        DictObj sysRoleMenu = new DictObj("SysRoleMenu", AppCodes.SysApp);
        sysRoleMenu.addCondition("roleId", roleId);
        sysRoleMenu.search();
        return sysRoleMenu.getRows().stream().map(r -> r.get("menuId").getLongValue()).collect(Collectors.toList());
    }

    private void copyRight(IDataAdapter adapter, long srcRole, long destRole, boolean autoCommit) {
        List<Long> roleMenus = getRoleMenus(destRole);
        DictObj sysRoleMenu = new DictObj("SysRoleMenu", AppCodes.SysApp);
        sysRoleMenu.addCondition("roleId", srcRole);
        sysRoleMenu.search();
        List<DictObjRow> rows = new ArrayList<>(sysRoleMenu.getRows());
        rows.forEach(r -> {
            Long menuId = r.get("menuId").getLongValue();
            if (roleMenus.contains(menuId)) { return; }

            DictObjRow newRow = sysRoleMenu.addRow();
            newRow.setValue("roleId", destRole);
            newRow.setValue("menuId", menuId);
        });

        if (!sysRoleMenu.save(adapter)) {
            String errMsg = sysRoleMenu.getErrors().stream()
                    .map(Error::getMessage)
                    .reduce(StringUtils.EMPTY, (s, error) -> s + StringUtils.LF + error);
            throw new BuzException(errMsg);
        }

        DictObj sysRoleAction = new DictObj("SysRoleAction", AppCodes.SysApp);
        sysRoleAction.addCondition("roleId", srcRole);
        sysRoleAction.search();
        List<DictObjRow> actionRows = new ArrayList<>(sysRoleAction.getRows());
        actionRows.forEach(r -> {

            Long menuId = r.get("menuId").getLongValue();
            if (roleMenus.contains(menuId)) { return; }

            DictObjRow newRow = sysRoleAction.addRow();
            newRow.setValue("roleId", destRole);
            newRow.setValue("menuId", menuId);
            newRow.setValue("actionId", r.getValue("actionId"));
        });

        if (!sysRoleAction.save(adapter)) {
            String errMsg = sysRoleAction.getErrors().stream()
                    .map(Error::getMessage)
                    .reduce(StringUtils.EMPTY, (s, error) -> s + StringUtils.LF + error);
            throw new BuzException(errMsg);
        }

        if (autoCommit) {
            adapter.commit();
        }
    }

    @Override
    public boolean BeforeDictObjSaveAsync(IDataAdapter adapter, MenuStore dict, DictObj obj) {
        if (StringUtils.equals(obj.getCode(), OBJ_ROLE) && dict.getState().containsKey("cloneRoleId")) {
            String cloneRole = dict.getState().get("cloneRoleId");
            long cloneRoleId = TypeUtil.toLong(cloneRole);
            obj.iterateInsertRows(r -> {
                copyRight(adapter, cloneRoleId, r.get("id").getLongValue());
                return true;
            });
        }

        if (StringUtils.equals(obj.getCode(), OBJ_ROLE)) {
            obj.getChanges().forEach(r -> {
                if (r.getState() == RowState.DELETED) {

                    // 删除角色用户
                    HSqlCommand cmd = new HSqlCommand("DELETE FROM sys_role_user where role_id=:roleId");
                    cmd.setParameter("roleId", r.getValue("id"));
                    adapter.update(cmd);

                    // 删除角色操作
                    cmd.resetSql();
                    cmd.appendSql("DELETE FROM sys_role_action where role_id=:roleId");
                    adapter.update(cmd);

                    // 删除角色菜单
                    cmd.resetSql();
                    cmd.appendSql("DELETE FROM sys_role_menu where role_id=:roleId");
                    adapter.update(cmd);

                }
            });

            return true;
        }

        return super.BeforeDictObjSaveAsync(adapter, dict, obj);
    }
}