package top.daxian1218.dust.edp.services;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.data.HDataTable;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.edp.BuzException;

import java.util.Map;

public abstract class BaseQueryService extends BuzService {
    public Object InvokeAsync(JSONObject inParams) {
        if (!BeforeQuery(getDataAdapter(), inParams)) {
            throw new BuzException("服务器校验失败，请检查参数");
        }

        HSqlCommand sqlCommand = new HSqlCommand();
        sqlCommand.appendSql(GetSql(inParams));
        sqlCommand.appendWhere(GetWhere(inParams));
        Map<String, Object> ps = GetParameters(inParams);
        if (ps != null) {
            sqlCommand.appendParameters(ps);
        }

        //分页信息
        if (inParams.containsKey("pageSize")) {
            sqlCommand.setPageIndex(inParams.getInteger("start"));
            sqlCommand.setPageSize(inParams.getInteger("pageSize"));
            sqlCommand.setTotalRows(0);
        }

        //约束条件
        if (inParams.containsKey("condition")) {
            JSONArray condition = inParams.getJSONArray("condition");
            for (Object item : condition) {
                JSONObject jo = (JSONObject) item;
                String field = jo.getString("field");
                String value = jo.getString("value");
                if (StringUtils.isEmpty(field)) {
                    throw new BuzException("field is null");
                }

                if (StringUtils.isEmpty(value)) {
                    continue;
                }

                char ch = field.charAt(0);
                String col = field.substring(1);
                StringBuilder sbColumnSql = new StringBuilder();
                switch (ch) {
                    case ')':
                        sbColumnSql.append(col);
                        sbColumnSql.append(">");
                        sbColumnSql.append("${INDEX}");
                        sqlCommand.appendParameter(GetValue(col, value));
                        break;
                    case '(':
                        sbColumnSql.append(col);
                        sbColumnSql.append("<");
                        sbColumnSql.append("${INDEX}");
                        sqlCommand.appendParameter(GetValue(col, value));
                        break;
                    case ']':
                        sbColumnSql.append(col);
                        sbColumnSql.append(">=");
                        sbColumnSql.append("${INDEX}");
                        sqlCommand.appendParameter(GetValue(col, value));
                        break;
                    case '[':
                        sbColumnSql.append(col);
                        sbColumnSql.append("<=");
                        sbColumnSql.append("${INDEX}");
                        sqlCommand.appendParameter(GetValue(col, value));
                        break;
                    case '=':
                        sbColumnSql.append(col);
                        sbColumnSql.append("=");
                        sbColumnSql.append("${INDEX}");
                        sqlCommand.appendParameter(GetValue(col, value));
                        break;
                    case '!':
                        sbColumnSql.append(col);
                        sbColumnSql.append("<>");
                        sbColumnSql.append("${INDEX}");
                        sqlCommand.appendParameter(GetValue(col, value));
                        break;
                    case '%':
                        sbColumnSql.append(col);
                        sbColumnSql.append(" LIKE ");
                        sbColumnSql.append("${INDEX}");
                        sqlCommand.appendParameter("%" + value + "%");
                        break;
                    case '@':
                        sbColumnSql.append(col);
                        sbColumnSql.append(" IN (");
                        String[] values = value.split(",");
                        for (String val : values) {
                            sbColumnSql.append("${INDEX},");
                            sqlCommand.appendParameter(val);
                        }
                        if (sbColumnSql.charAt(sbColumnSql.length() - 1) == ',') {
                            sbColumnSql = sbColumnSql.replace(sbColumnSql.length() - 1, sbColumnSql.length(), "");
                        }

                        sbColumnSql.append(")");
                        break;
                    default:
                        sbColumnSql.append(col);
                        sbColumnSql.append("=");
                        sbColumnSql.append("${INDEX}");
                        sqlCommand.appendParameter(GetValue(field, value));
                        break;
                }

                String subTableSql = GetExistsSubTableSql(inParams, col);
                if (subTableSql != null) {
                    sbColumnSql.insert(0, subTableSql + " AND ");
                    sbColumnSql.insert(0, "EXISTS(");
                    sbColumnSql.append(")");
                }

                sqlCommand.appendWhere(sbColumnSql.toString());
            }
        }

        //排序
        if (inParams.containsKey("orders")) {
            String orders = inParams.getString("orders");
            if (!StringUtils.isEmpty(orders)) {
                sqlCommand.appendOrderString(orders);
            }
        }

        HDataTable dt = getDataAdapter().query(sqlCommand);
        return QueryResult(sqlCommand, dt.toDataJSON());
    }

    protected Object QueryResult(HSqlCommand sqlCommand, JSONArray rows) {
        if (sqlCommand.getPageSize() > 0) {
            JSONObject result = new JSONObject();
            result.put("size", sqlCommand.getTotalRows());
            result.put("page", rows);
            return result;
        }

        return rows;
    }

    /**
     * 根据字段获取实际值
     *
     * @param field
     * @param val
     * @return
     */
    public abstract Object GetValue(String field, String val);

    /**
     * 获取执行Sql，如果包含where，请勿写在GetSql中。推荐where写到GetWhere
     *
     * @param inParams
     * @return
     */
    public abstract String GetSql(JSONObject inParams);

    /**
     * 获取Where条件，SQL
     *
     * @param inParams
     * @return
     */
    public abstract String GetWhere(JSONObject inParams);

    public Map<String, Object> GetParameters(JSONObject inParams) {
        return null;
    }

    public String GetExistsSubTableSql(JSONObject inParams, String col) {
        return null;
    }

    public boolean BeforeQuery(IDataAdapter adapter, JSONObject inParams) {
        return true;
    }
}