package top.daxian1218.dust.edp.services;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import top.daxian1218.dust.commons.util.BeanUtil;
import top.daxian1218.dust.db.DictGlobalConfig;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.edp.BuzException;

@Component
public class BuzServiceManager
{

	public Object ExecuteAsync(JSONObject inParams)
	{
		if (!inParams.containsKey("method"))
		{
			throw BuzException.ArgumentMissing("method");
		}

		String method = inParams.getString("method");
		if (StringUtils.isEmpty(method))
		{
			throw BuzException.ArgumentMissing("method");
		}

		IDataAdapter adapter = GetDataAdapter(inParams);
		JSONObject body = inParams.getJSONObject("params");

		return ExecutePluginAsync(method, adapter, body);

	}

	public Object ExecutePluginAsync(String method, IDataAdapter adapter, JSONObject body)
	{
		Object bean = BeanUtil.getBean(method);
		if (bean instanceof IBuzService)
		{
			IBuzService buzService = (IBuzService) bean;
			buzService.setDataAdapter(adapter);
			return buzService.InvokeAsync(body);
		}

		throw new BuzException("method(" + method + ") not found");
	}

	private IDataAdapter GetDataAdapter(JSONObject inParams)
	{
		if (inParams.containsKey("$dataAdapter"))
		{
			return (IDataAdapter)inParams.get("$dataAdapter");
		}

		return DictGlobalConfig.Instance.getDataAdapter();
	}
}