package top.daxian1218.dust.edp.services.dev;

import com.alibaba.fastjson.JSONObject;
import org.springframework.stereotype.Service;
import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.db.dict.condition.OperationType;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.services.BuzService;

import java.util.Map;

@Service
public class DefaultConditionSvc extends BuzService {
    public Object InvokeAsync(JSONObject inParams) {
        String action = inParams.getString("action");
        if (action == null) {
            return false;
        }

        switch (action) {
            case Actions.Where:
                HandleWhere(inParams);
                break;
            case Actions.BeforeSearch:
                BeforeSearch(inParams);
                break;
            case Actions.AfterSearch:
                AfterSearch(inParams);
                break;
            default:
                throw new BuzException("unsupported action");
        }

        return true;
    }

    private void AfterSearch(JSONObject inParams) {

    }

    private void BeforeSearch(JSONObject inParams) {

    }

    private void HandleWhere(JSONObject inParams) {
        DictObj obj = (DictObj) inParams.get("obj");
        if (obj == null) {
            return;
        }

        //默认查询，用于字典配置列查询
        JSONObject defaultQuery = inParams.getJSONObject("defaultQuery");
        if (defaultQuery != null) {
            for (Map.Entry<String, Object> kv : defaultQuery.entrySet()) {
                obj.addCondition(new DictCondition(kv.getKey(), kv.getValue()));
            }
        }

        //固定查询，用于主从查询或者当行查询操作
        JSONObject fixQuery = inParams.getJSONObject("fixQuery");
        if (fixQuery != null) {
            fixQuery.forEach((k, v) -> {
                obj.addCondition(new DictCondition(k, v));
            });
        }

        //关键字查询
        JSONObject keywordQuery = inParams.getJSONObject("keywordQuery");
        if (keywordQuery != null) {
            DictCondition keywordCondition = null;
            for(Map.Entry<String, Object> kv : keywordQuery.entrySet())
            {
                if (keywordCondition == null) {
                    keywordCondition = new DictCondition(kv.getKey(), kv.getValue());
                    keywordCondition.setOperation(OperationType.CONTAIN);
                } else {

                    DictCondition subCondition = new DictCondition(kv.getKey(), kv.getValue());
                    subCondition.setOperation(OperationType.CONTAIN);
                    subCondition.setRequire(false);

                    keywordCondition.getSubConditions().add(subCondition);
                }
            }

            obj.addCondition(keywordCondition);
        }

        //TODO 高级查询
    }

    private static class Actions {
        public static final String Where = "Where";
        public static final String BeforeSearch = "BeforeSearch";
        public static final String AfterSearch = "AfterSearch";
    }
}