package top.daxian1218.dust.edp.services.dev;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import top.daxian1218.dust.db.DictGlobalConfig;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.dict.*;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.db.dict.condition.OperationType;
import top.daxian1218.dust.db.dict.condition.expressions.ColumnExpression;
import top.daxian1218.dust.db.dict.condition.expressions.ValueListExpression;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.edp.AppCodes;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.DictCodes;
import top.daxian1218.dust.edp.services.BaseService;
import top.daxian1218.dust.edp.utils.DictObjUtil;
import top.daxian1218.dust.edp.utils.JsonUtil;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class DictManagerService extends BaseService {
    public DictObj ListAsync(String alias, DictCondition condition, PageInfo pageInfo) {
        DictObj obj = DictObjBuilder.create(alias, AppCodes.DictApp);
        if (condition != null) {
            obj.addCondition(condition);
        }

        if (pageInfo != null) {
            obj.getPageInfo().setPageSize(pageInfo.getPageSize());
            obj.getPageInfo().setStart(pageInfo.getStart());
        }

        OrderBy orderBy = new OrderBy();
        orderBy.setColumnName("id");
        orderBy.setOrderByType(OrderByType.ASC);

        obj.addOrderBy(orderBy);
        obj.search();
        return obj;
    }

    public DictObjRow Find(String objCode, String id) {
        DictCondition cdt = new DictCondition("id", id);
        return (ListAsync(objCode, cdt, null)).getFirstOrDefaultRow();
    }

    public DictObjRow SaveAsync(String alias, JSONObject data) {
        DictObj obj = DictObjBuilder.create(alias, AppCodes.DictApp);
        JsonUtil.JsonToRow(data, obj);
        if (!obj.validate()) {
            throw new BuzException("保存数据校验失败，请检查必填等");
        }

        obj.save();
        getDefaultAdapter().commit();

        long id = obj.getRow(0).get("id").getLongValue();
        obj.search(id);
        return obj.getFirstOrDefaultRow();
    }

    public boolean DeleteAsync(String alias, String id) {
        DictObj obj = DictObjBuilder.create(alias, AppCodes.DictApp);
        HSqlCommand cmd = new HSqlCommand("DELETE FROM " + obj.getTableName());
        cmd.appendWhere("id=:id");
        cmd.setParameter("id", id);
        DictGlobalConfig.Instance.getDataAdapter().update(cmd);
        DictGlobalConfig.Instance.getDataAdapter().commitAndCloseQuiet();
        return true;
    }

    public boolean DeleteCascadeAsync(String alias, String id, String... objNames) {
        DictObj obj = DictObjBuilder.create(alias, AppCodes.DictApp);
        HSqlCommand cmd = new HSqlCommand("DELETE FROM " + obj.getTableName());
        cmd.appendWhere("id=:id");
        cmd.setParameter("id", id);
        DictGlobalConfig.Instance.getDataAdapter().update(cmd);
        if (objNames != null && objNames.length > 0) {
            for (String objCode : objNames) {
                DictObj cascadeObj = DictObjBuilder.create(alias, AppCodes.DictApp);
                HSqlCommand delByPidCommand = new HSqlCommand("DELETE FROM " + cascadeObj.getTableName());
                delByPidCommand.appendWhere("pid=:id");
                delByPidCommand.setParameter("pid", id);
                DictGlobalConfig.Instance.getDataAdapter().update(cmd);
            }
        }
        DictGlobalConfig.Instance.getDataAdapter().commitAndCloseQuiet();
        return true;
    }

    public boolean DeleteObj(String id) {
        IDataAdapter adapter = DictGlobalConfig.Instance.getDataAdapter();
        DictObj obj = DictObjBuilder.create("DictObj", AppCodes.DictApp);
        obj.addCondition(new DictCondition(obj.getTableName(), "id", id));
        obj.search();
        if (obj.getRows().size() == 0) {
            return true;
        }

        DictObjRow row = obj.getFirstOrDefaultRow();
        HSqlCommand colCmd = new HSqlCommand("DELETE FROM dict_obj_column WHERE obj_code=:objCode AND app_code=:appCode");
        colCmd.setParameter("objCode", row.get("code").getValue());
        colCmd.setParameter("appCode", row.get("appCode").getValue());
        adapter.update(colCmd);

        HSqlCommand tableCmd = new HSqlCommand("DELETE FROM dict_obj_table WHERE obj_code=:objCode AND app_code=:appCode");
        tableCmd.setParameter("objCode", row.get("code").getValue());
        tableCmd.setParameter("appCode", row.get("appCode").getValue());
        adapter.update(colCmd);

        HSqlCommand cmd = new HSqlCommand("DELETE FROM dict_obj WHERE id=:id");
        cmd.setParameter("id", id);
        adapter.update(cmd);
        adapter.commitAndCloseQuiet();
        return true;
    }

    public String GetSchemaJSON(String appCode, String objCode) {
        DictObj obj = DictObjBuilder.create(objCode, appCode);
        return obj.toJson().toString();
    }

    public String GetSchemaJSON(String id) {
        //TODO support json
        return "";
    }

    boolean CloneColumnsAsync(String srcId, String[] colIds, String destId) {

        DictObjRow srcObjRow = Find("dict_obj", srcId);
        if (srcObjRow == null) {
            throw new BuzException("源对象不存在");
        }

        DictObjRow destObjRow = Find("dict_obj", destId);
        if (destObjRow == null) {
            throw new BuzException("目标对象不存在");
        }

        DictCondition idInCondition = new DictCondition();
        idInCondition.setLeft(new ColumnExpression("id"));
        idInCondition.setOperation(OperationType.IN_LIST);
        idInCondition.setRight(new ValueListExpression(Arrays.asList(colIds.clone())));

        DictObj objColumns = ListAsync("dict_obj_column", idInCondition, null);

        if (objColumns.getRows().size() == 0) {
            return true;
        }

        DictObj objDest = new DictObj("dict_obj_column", AppCodes.DictApp);
        objDest.addCondition(new DictCondition("objId", destId));
        objDest.search();

        List<String> rowLabels = objDest.getRows().stream()
                .map(r -> r.getField("columnLabel").getStringValue())
                .collect(Collectors.toList());

        List<String> tableNames = new ArrayList<>();
        objColumns.getRows().forEach(row ->
        {
            String label = row.getField("columnLabel").getStringValue();
            if (rowLabels.contains(label)) {
                return;
            }

            if (StringUtils.equals(row.get("objId").getStringValue(), srcId)) {
                DictObjRow newRow = objDest.newRow();
                String tableName = row.get("tableName").getStringValue();
                String srcTableName = srcObjRow.get("tableName").getStringValue();
                String objTableName = destObjRow.get("tableName").getStringValue();

                // 如果关联表，则缓存，用于后续处理
                if (!StringUtils.equalsIgnoreCase(tableName, objTableName)) {
                    tableNames.add(tableName);
                }

                DictObjUtil.Clone(row, newRow);

                // 如果表名为空，则默认为对象表
                if (StringUtils.isEmpty(tableName) || StringUtils.equalsIgnoreCase(srcTableName, tableName)) {
                    newRow.get("tableName").setValue(objTableName);
                }

                newRow.get("objId").setValue(destObjRow.get("id").getValue());
                newRow.get("appCode").setValue(destObjRow.get("appCode").getValue());
                newRow.get("objCode").setValue(destObjRow.get("code").getValue());
                objDest.addRow(newRow);
            }
        });
        CloneTable(tableNames, srcObjRow, destObjRow);
        objDest.save();

        return true;
    }

    private void CloneTable(List<String> tableNames, DictObjRow src, DictObjRow dest) {
        if (tableNames.size() == 0) return;

        List<String> names = tableNames.stream().distinct().collect(Collectors.toList());

        DictObj objTable = new DictObj("DictObjTable", AppCodes.DictApp);
        objTable.addCondition(new DictCondition("objId", src.get("id").getValue()));
        objTable.search();

        //检索所有受影响关联表
        String srcTableName = src.get("tableName").getStringValue();
        Map<String, DictObjRow> srcTableMap = ScanTables(tableNames, objTable, srcTableName, null);

        objTable.addCondition(new DictCondition("objId", dest.get("id").getValue()));
        objTable.search();

        objTable.getRows().forEach(r ->
        {
            String label = r.get("tableAlias").getStringValue();
            if (srcTableMap.containsKey(label)) {
                srcTableMap.remove(label);
            }
        });

        for(DictObjRow r : srcTableMap.values())
        {
            DictObjRow newRow = objTable.newRow();
            DictObjUtil.Clone(r, newRow);
            newRow.get("objId").setValue(dest.get("id").getValue());
            newRow.get("appCode").setValue(dest.get("appCode").getValue());
            newRow.get("objCode").setValue(dest.get("code").getValue());
            objTable.addRow(newRow);
        }

        objTable.save();
    }

    /**
     * 检索需要复制的表结构
     *
     * @param labels
     * @param src
     * @param srcTable
     * @param foundTables
     * @return
     */
    private Map<String, DictObjRow> ScanTables(List<String> labels, DictObj src, String srcTable, List<String> foundTables) {
        Map<String, DictObjRow> result = new HashMap<>();
        if (foundTables == null) {
            foundTables = new ArrayList<>();
        }

        List<String> nextLabels = new ArrayList<>();
        Map<String, DictObjRow> finalResult = result;
        List<String> finalFoundTables = foundTables;
        src.getRows().forEach(r ->
        {
            String label = r.get("tableAlias").getStringValue();
            if (finalResult.containsKey(label)) {
                return;
            }

            if (labels.contains(label)) {
                finalResult.put(label, r);
                finalFoundTables.add(label);

                // 查找关系链
                String relationTable = r.get("relationTable").getStringValue();
                if (!StringUtils.isEmpty(relationTable)
                        && !StringUtils.equalsIgnoreCase(srcTable, relationTable)) {
                    nextLabels.add(relationTable);
                }
            }
        });
        if (nextLabels.size() > 0) {
            result.putAll(ScanTables(nextLabels, src, srcTable, foundTables));
        }

        return result;
    }

    public Object UpdateColumnUISettingAsync(String id, JSONObject body) {
        DictObjRow row = Find("dict_obj_column", id);
        if (row == null) {
            throw new BuzException("行不存在");
        }

        DictObj dataDict = new DictObj(DictCodes.DictData, AppCodes.DictApp);

        DictObjRow settingRow;
        if (row.get("uiSettings") == null || row.get("uiSettings").getInt32Value() == 0) {
            settingRow = dataDict.addRow();
        } else {
            dataDict.addCondition(new DictCondition("id", row.get("uiSettings").getValue()));
            dataDict.search(getDefaultAdapter());
            if (dataDict.getRows().size() > 0) {
                settingRow = dataDict.getRow(0);
            } else {
                settingRow = dataDict.addRow();
            }
        }

        settingRow.get("code").setValue("dict_obj_column:id:" + row.get(" id ").getStringValue());
        settingRow.get("value").setValue(body.toString());
        settingRow.get("category").setValue(DataDictType.OBJ_PROPS.getValue());

        dataDict.save(getDefaultAdapter());


        row.get("uiSettingId").setValue(settingRow.get("id").getInt32Value());
        row.getDictObj().save(getDefaultAdapter());
        getDefaultAdapter().commit();

        return Find("dict_obj_column", id);
    }
}