package top.daxian1218.dust.edp.services.dict;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.DictObjRow;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.edp.AppCodes;
import top.daxian1218.dust.edp.DictCodes;
import top.daxian1218.dust.edp.services.BuzService;

/**
 * @author daxian
 */
@Service("dictMigration")
public class DictMigrationService extends BuzService {
    @Override
    public Object InvokeAsync(JSONObject inParams) {
        expandDictObjOfUIObj();
        return true;
    }

    private void expandDictObjOfUIObj() {
        IDataAdapter adapter = getDataAdapter();
        DictObj objDict = new DictObj(DictCodes.DictObj, AppCodes.DictApp);
        DictObj objColumn = new DictObj(DictCodes.DictObjColumn, AppCodes.DictApp);
        DictObj objTable = new DictObj(DictCodes.DictObjTable, AppCodes.DictApp);
        DictObj uiDict = new DictObj(DictCodes.DictUIObj, AppCodes.DictApp);
        DictObj uiColumn = new DictObj(DictCodes.DictUIObjColumn, AppCodes.DictApp);
        DictObj columnUI = new DictObj(DictCodes.DictObjColumnUI, AppCodes.DictApp);
        //DictObj newDict = new DictObj(DictCodes.DictObj, AppCodes.DictApp);

        // 检索遍历所有界面对象
        uiDict.search(adapter);
        if (uiDict.isEmpty()) {
            return;
        }

        int[] seq = new int[] { 1 };



        uiDict.getRows().forEach(r -> {
            long objId = r.get("objId").getLongValue();

            objDict.search(objId);
            if (objDict.isEmpty()) { return; }

            DictObjRow objRow = objDict.getRow(0);
            DictObjRow newRow = objDict.addRow();

            objDict.getColumns().forEach(col -> {
                if (StringUtils.equals("id", col.getPhysicalName())) {
                    return;
                }

                newRow.setValue(col.getColumnName(), objRow.getValue(col.getColumnName()));
            });

            String newCode = "U" + StringUtils.leftPad(String.valueOf(seq[0]), 6, "0");
            seq[0]++;

            newRow.setValue("code", newCode);
            objDict.save(adapter);

            long newId = newRow.getId();
            r.setValue("objId", newId);


            cloneDetail(objColumn, objId, newId, newRow.get("code").getStringValue(), objRow.get("code").getStringValue());
            cloneDetail(objTable, objId, newId);

            uiColumn.addCondition("objId", r.getId());
            uiColumn.search(adapter);
            uiColumn.getRows().forEach(uiCol -> {
                DictObjRow newColRow = columnUI.addRow();
                columnUI.getColumns().forEach(col -> {
                    if (StringUtils.equals("id", col.getPhysicalName())) {
                        return;
                    }
                    newColRow.setValue(col.getColumnName(), uiCol.getValue(col.getColumnName()));
                });

                newColRow.setValue("objId", newId);
            });

        });

        uiDict.save(adapter);
        columnUI.save(adapter);

        HSqlCommand cmd = new HSqlCommand("UPDATE \n" +
                "\tDICT_OBJ_COLUMN_UI\n" +
                "SET \n" +
                "\tCOLUMN_ID = (\n" +
                "\t\n" +
                "\tSELECT newID FROM \n" +
                "\t(SELECT\n" +
                "\tdocu.id AS NID,\n" +
                "\tdocu.OBJ_ID AS objid,\n" +
                "\tdocu.COLUMN_ID AS oldId,\n" +
                "\tdoc2.id AS newID\n" +
                "FROM\n" +
                "\tDICT_OBJ_COLUMN_UI docu\n" +
                "LEFT JOIN DICT_OBJ_COLUMN doc ON\n" +
                "\tdoc.id = docu.COLUMN_ID\n" +
                "LEFT JOIN DICT_OBJ_COLUMN doc2 ON\n" +
                "\tdoc2.COLUMN_NAME = doc.COLUMN_NAME\n" +
                "\tAND doc2.OBJ_ID = docu.OBJ_ID) t\n" +
                "\tWHERE t.NID=ID\n" +
                "\t)");
        adapter.update(cmd);

        adapter.commit();
    }

    private void cloneDetail(DictObj obj, long objId, long newId, String... ps) {
        obj.addCondition("objId", objId);
        obj.search(getDataAdapter());
        for (int i = obj.getRows().size() - 1; i >= 0; i--) {
            DictObjRow colRow = obj.getRow(i);
            DictObjRow newColRow = obj.addRow();
            obj.getColumns().forEach(col -> {
                if (StringUtils.equals("id", col.getPhysicalName())) {
                    return;
                }
                newColRow.setValue(col.getColumnName(), colRow.getValue(col.getColumnName()));
            });

            if (StringUtils.equals(obj.getCode(), DictCodes.DictObjColumn) &&
            StringUtils.equals(ps[1], colRow.get("tableName").getStringValue())) {
                newColRow.setValue("tableName", ps[0]);
            }

            newColRow.setValue("objId", newId);
        }

        obj.save(getDataAdapter());
    }
}
