package top.daxian1218.dust.edp.services.dict;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.dict.*;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.db.dict.condition.OperationType;
import top.daxian1218.dust.db.dict.condition.expressions.ColumnExpression;
import top.daxian1218.dust.db.dict.condition.expressions.ValueExpression;
import top.daxian1218.dust.db.dict.condition.expressions.ValueListExpression;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.services.BuzService;

import java.util.Arrays;

/**
 * @author daxian
 */
@Service
public class DictQueryService extends BuzService {
    public Object InvokeAsync(JSONObject inParams) {
        if (inParams == null) {
            throw new BuzException("query method's inParams not found");
        }

        String objCode = inParams.getString("objCode");
        if (StringUtils.isEmpty(objCode)) {
            throw new BuzException("objCode is null");
        }

        DictObj obj = DictObjBuilder.create(objCode, null);

        //分页信息
        if (inParams.containsKey("pageSize")) {
            obj.getPageInfo().setPageSize(inParams.getInteger("pageSize"));
            obj.getPageInfo().setStart(inParams.getInteger("start"));
        }

        //约束条件
        if (inParams.containsKey("condition")) {
            JSONArray condition = inParams.getJSONArray("condition");
            for (Object item : condition) {
                JSONObject jo = (JSONObject) item;
                String field = jo.getString("field");
                String value = jo.getString("value");
                if (StringUtils.isEmpty(value)) {
                    continue;
                }

                if (StringUtils.isEmpty(field)) {
                    throw new BuzException("field is null");
                }

                char ch = field.charAt(0);
                DictCondition where = new DictCondition();
                String colName = field.substring(1);

                //不存在的条件，可触发自定义
                boolean hasColumn = obj.getColumn(colName) != null;
                if (!hasColumn) {
                    DictCondition subWhere = GetCustomCondition(obj, colName, value);
                    if (subWhere != null) {
                        obj.addCondition(subWhere);
                    }
                    continue;
                }

                DictObjColumn col = obj.getColumn(colName);
                String tableName = StringUtils.isEmpty(col.getTableName()) ? obj.getTableName() : col.getTableName();
                where.setLeft(new ColumnExpression(tableName, col.getColumnName()));
                ValueExpression valueNode = new ValueExpression(value);
                valueNode.setDataType(col.getDataType());

                where.setRight(valueNode);
                switch (ch) {
                    case ')':
                        where.setOperation(OperationType.GREATER);
                        break;
                    case '(':
                        where.setOperation(OperationType.LESS);
                        break;
                    case ']':
                        where.setOperation(OperationType.EQUAL_GREATER);
                        break;
                    case '[':
                        where.setOperation(OperationType.EQUAL_LESS);
                        break;
                    case '#':
                        where.setOperation(TypeUtil.toBoolean(valueNode.getValue()) ?
                                OperationType.NOT_NULL : OperationType.NULL);
                        break;
                    case '=':
                        where.setOperation(OperationType.EQUAL);
                        break;
                    case '!':
                        where.setOperation(OperationType.NOT_EQUAL);
                        break;
                    case '%':
                        where.setOperation(OperationType.CONTAIN);
                        break;
                    case '^':
                        where.setOperation(OperationType.START_WITH);
                        break;
                    case '$':
                        where.setOperation(OperationType.END_WITH);
                        break;
                    case '@':
                        where.setOperation(OperationType.IN_LIST);
                        ValueListExpression valueListNode = new ValueListExpression(Arrays.asList(value.split(",").clone()));
                        valueListNode.setDataType(col.getDataType());
                        where.setRight(valueListNode);
                        break;
                    default:
                        where.setLeft(new ColumnExpression(obj.getTableName(), field));
                        where.setOperation(OperationType.EQUAL);
                        break;
                }
                obj.addCondition(where);
            }
        }

        //排序
        if (inParams.containsKey("orders")) {
            String orders = inParams.getString("orders");
            if (!StringUtils.isEmpty(orders)) {
                String[] arrOrder = orders.split(",");
                for (String item : arrOrder) {
                    String[] arr = item.split(" ");
                    OrderBy orderBy = new OrderBy(arr[0]);
                    OrderByType type = OrderByType.ASC;
                    if (arr.length == 2) {
                        type = StringUtils.equalsIgnoreCase(arr[1], "desc") ? OrderByType.DESC : OrderByType.ASC;
                    }


                    orderBy.setOrderByType(type);
                    obj.addOrderBy(orderBy);
                }

            }
        }

        if (BeforeQuery(obj, inParams)) {
            obj.search(getDataAdapter());
            AfterQuery(obj, inParams);
        }

        return obj.getPageInfo().getPageSize() > 0 ? obj.toPageDataJSON() : obj.toDataJSON();
    }

    protected boolean BeforeQuery(DictObj dataObj, JSONObject inParams) {
        return true;
    }

    protected void AfterQuery(DictObj dataObj, JSONObject inParams) {
    }

    protected DictCondition GetCustomCondition(DictObj dataObj, String colName, String value) {
        return null;
    }
}