package top.daxian1218.dust.edp.services.queryProject;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import top.daxian1218.dust.db.dict.*;
import top.daxian1218.dust.db.dict.Error;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.db.dict.condition.OperationType;
import top.daxian1218.dust.db.dict.condition.expressions.ColumnExpression;
import top.daxian1218.dust.db.dict.condition.expressions.ValueListExpression;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.services.BaseService;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class AdvancedQueryService extends BaseService {
    /**
     * 获取查询选项
     *
     * @param objId
     * @return
     */
    public JSONObject listProjectItems(String menuId, String objId) {
        if (StringUtils.isEmpty(menuId)) {
            throw new BuzException("无效的参数menuId");
        }

        if (StringUtils.isEmpty(objId)) {
            throw new BuzException("无效的参数objId");
        }

        DictObj queryProject = new DictObj("QueryProject", "dict");
        DictObj queryProjectColumn = new DictObj("QueryProjectItem", "dict");

        queryProject.addCondition("menuId", menuId);
        queryProject.addCondition("objId", objId);

        long userId = getUserId();
        DictCondition type = new DictCondition("projectType", 1);
        type.setRequire(false);
        type.setOperation(OperationType.NOT_EQUAL);

        DictCondition where = new DictCondition();
        where.getSubConditions().add(new DictCondition("createBy", userId));
        where.getSubConditions().add(type);
        queryProject.addCondition(where);

        queryProject.search(getDefaultAdapter());
        if (queryProject.isEmpty()) {
            JSONObject result = new JSONObject();
            result.put("items", new JSONArray());
            return result;
        }

        DictCondition whereItem = new DictCondition();
        whereItem.setLeft(new ColumnExpression("projectId"));
        whereItem.setOperation(OperationType.IN_LIST);
        whereItem.setRight(new ValueListExpression(queryProject.getRows().stream().map(r -> r.getLongValue("id")).collect(Collectors.toList())));
        queryProjectColumn.addCondition(whereItem);
        OrderBy orderBy = new OrderBy();
        orderBy.setColumnName("projectId");
        queryProjectColumn.addOrderBy(orderBy);
        orderBy = new OrderBy();
        orderBy.setColumnName("id");
        orderBy.setOrderByType(OrderByType.DESC);
        queryProjectColumn.addOrderBy(orderBy);

        queryProjectColumn.search(getDefaultAdapter());

        if (queryProject.getRows().size() == 1) {
            JSONObject result = new JSONObject();
            result.put("items", queryProjectColumn.toDataJSON());
            return result;
        }

        final long[] projectIds = new long[]{0, 0, 0};
        for (DictObjRow row : queryProject.getRows()) {
            int projectType = row.getInt32Value("projectType");
            if (projectType > 2 || projectType < 0) {
                continue;
            }

            projectIds[projectType] = row.getLongValue("id");
        }

        Map<String, JSONObject> queryItems = new HashMap<>(queryProject.getRows().size());

        queryProjectColumn.getRows().forEach(r -> {
            String itemName = r.getStringValue("itemName");
            long projectId = r.getLongValue("projectId");
            int projectType = ArrayUtils.indexOf(projectIds, projectId);

            if (queryItems.containsKey(itemName)) {
                JSONObject existItem = queryItems.get(itemName);
                if (existItem.getIntValue("projectType") < projectType) {
                    return;
                }
            }

            JSONObject item = r.toJSON();
            item.put("projectType", projectType);
            queryItems.put(itemName, item);
        });


        JSONObject result = new JSONObject();
        result.put("items", new ArrayList<>(queryItems.values()));
        return result;
    }

    private void addProject(DictObj queryProject, String menuId, String objId) {
        DictObjRow newRow = queryProject.addRow();
        newRow.setValue("menuId", menuId);
        newRow.setValue("projectName", "默认");
        newRow.setValue("projectType", 1);
        newRow.setValue("objId", objId);
        newRow.setValue("createBy", getUserId());
        if (!queryProject.save()) {
            throw new BuzException(queryProject.getErrors().stream().map(Error::getMessage).collect(Collectors.joining()));
        }
    }

    /**
     * 获取查询方案
     *
     * @param menuId 菜单ID
     * @param objId  对象ID
     * @return
     */
    public JSONObject getQueryProject(String menuId, String objId) {
        if (StringUtils.isEmpty(menuId)) {
            throw new BuzException("无效的参数menuId");
        }

        if (StringUtils.isEmpty(objId)) {
            throw new BuzException("无效的参数objId");
        }

        DictObj obj = new DictObj("QueryProject", "dict");
        obj.addCondition("menuId", menuId);
        obj.addCondition("objId", objId);
        obj.addCondition("createBy", getUserId());
        obj.search(getDefaultAdapter());
        if (obj.isEmpty()) {
            addProject(obj, menuId, objId);
            JSONObject result = obj.getRow(0).toJSON();
            result.put("items", new JSONArray());
            return result;
        }


        DictObj queryProjectColumn = new DictObj("QueryProjectItem", "dict");
        queryProjectColumn.addCondition("projectId", obj.getRow(0).getLongValue("id"));
        queryProjectColumn.search(getDefaultAdapter());

        JSONObject result = obj.getRow(0).toJSON();
        result.put("items", queryProjectColumn.toDataJSON());
        return result;

    }

    /**
     * 更新查询方案
     *
     * @param project 更新字段
     * @return
     */
    public JSONObject updateProject(JSONObject project) {
        if (project == null || project.isEmpty()) {
            throw new BuzException("保存方案失败，无效参数");
        }

        String projectId = project.getString("id");

        DictObj queryProject = new DictObj("QueryProject", "dict");
        queryProject.addCondition("id", projectId);
        queryProject.addCondition("createBy", getUserId());

        queryProject.search();
        if (queryProject.isEmpty()) {
            throw new BuzException("保存方案失败，当前用户无法修改");
        }

        DictObjRow row = queryProject.getRow(0);
        if (project.getBooleanValue("share")) {
            row.setValue("projectType", 2);
        } else {
            row.setValue("projectType", 1);
        }

        DictObj queryProjectColumn = new DictObj("QueryProjectItem", "dict");
        queryProjectColumn.addCondition("projectId", projectId);
        queryProjectColumn.search();
        queryProjectColumn.deleteAll();

        if (project.containsKey("items")) {
            JSONArray items = project.getJSONArray("items");
            for (Object i : items) {
                if (!(i instanceof JSONObject)) {
                    continue;
                }

                JSONObject jo = (JSONObject) i;
                DictObjRow newRow = queryProjectColumn.addRow();
                newRow.setValue("projectId", projectId);
                newRow.setValue("itemOrder", jo.getIntValue("itemOrder"));
                newRow.setValue("itemType", jo.getIntValue("itemType"));
                newRow.setValue("isRequired", jo.getIntValue("isRequired"));
                newRow.setValue("itemSrc", jo.getString("itemSrc"));
                newRow.setValue("itemName", jo.getString("itemName"));
                newRow.setValue("defaultValue", jo.getString("defaultValue"));
                newRow.setValue("remark", jo.getString("remark"));
            }
        }
        queryProject.save(getDefaultAdapter());
        queryProjectColumn.save(getDefaultAdapter());

        getDefaultAdapter().commit();

        return listProjectItems(row.getStringValue("menuId"), row.getStringValue("objId"));
    }
}