package top.daxian1218.dust.edp.services.storage;

import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
@ConfigurationProperties("edp.storage")
public class StorageOptions {
    /**
     * 默认存储，目前支持Mongo、Local
     */
    private StorageLocation location  = StorageLocation.LOCAL;

    /**
     * Mongo GridFS存储方案配置
     */
    private Mongo mongo = new Mongo();


    /**
     * 本地存放配置
     */
    public Local local = new Local();

    public StorageLocation getLocation() {
        return location;
    }

    public void setLocation(StorageLocation location) {
        this.location = location;
    }

    public Mongo getMongo() {
        return mongo;
    }

    public void setMongo(Mongo mongo) {
        this.mongo = mongo;
    }

    public Local getLocal() {
        return local;
    }

    public void setLocal(Local local) {
        this.local = local;
    }

    public static class Mongo {
        private String host;
        private List<String> slaves = new ArrayList<>();
        private String dataBase;
        private String userName;
        private String password;

        /**
         * 过期时间，单位秒，默认5分钟
         */
        private int expireAfter = 5 * 60;

        /**
         * 长过期时间，单位时，默认1天
         */
        private int longExpireAfter = 24;

        public String getHost() {
            return host;
        }

        public void setHost(String host) {
            this.host = host;
        }

        public List<String> getSlaves() {
            return slaves;
        }

        public String getDataBase() {
            return dataBase;
        }

        public void setDataBase(String dataBase) {
            this.dataBase = dataBase;
        }

        public int getExpireAfter() {
            return expireAfter;
        }

        public void setExpireAfter(int expireAfter) {
            this.expireAfter = expireAfter;
        }

        public int getLongExpireAfter() {
            return longExpireAfter;
        }

        public void setLongExpireAfter(int longExpireAfter) {
            this.longExpireAfter = longExpireAfter;
        }

        public String getUserName() {
            return userName;
        }

        public void setUserName(String userName) {
            this.userName = userName;
        }

        public String getPassword() {
            return password;
        }

        public void setPassword(String password) {
            this.password = password;
        }
    }

    public static class Local {
        private String filePath = "upload";

        public String getFilePath() {
            return filePath;
        }

        public void setFilePath(String filePath) {
            this.filePath = filePath;
        }
    }
}

