package top.daxian1218.dust.edp.ui;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import top.daxian1218.dust.commons.data.HDataTable;
import top.daxian1218.dust.commons.log.LoggerHelper;
import top.daxian1218.dust.commons.thread.LocalHolder;
import top.daxian1218.dust.commons.util.ClassUtil;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.DbAdapterManager;
import top.daxian1218.dust.db.DictGlobalConfig;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.dict.*;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.db.dict.condition.OperationType;
import top.daxian1218.dust.db.dict.condition.expressions.ColumnExpression;
import top.daxian1218.dust.db.dict.condition.expressions.ValueListExpression;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.edp.AppCodes;
import top.daxian1218.dust.edp.DictCodes;
import top.daxian1218.dust.edp.utils.DictObjUtil;

import java.util.*;
import java.util.stream.Collectors;

import static top.daxian1218.dust.edp.AppCodes.SysApp;

/**
 * @author daxian
 */
@Component
public class DictLoader {
    private final DbAdapterManager _dbAdapterManager;
    private static final String LOCAL_ADAPTER = "DictLoader";
    public static final String MENU_PREFIX = "DICT:SYS_MENU:ID:";
    public static final String DICT_DATA_KEY = "DICT:DICT_DATA";

    public DictLoader(DbAdapterManager dbAdapterManager) {
        _dbAdapterManager = dbAdapterManager;
    }

    private IDataAdapter getDictAdapter() {
        IDataAdapter dictAdapter = LocalHolder.get(LOCAL_ADAPTER);
        if (dictAdapter != null) {
            return dictAdapter;
        }

        dictAdapter = _dbAdapterManager.getAdapter(DictGlobalConfig.Instance.getDictDb());
        LocalHolder.set(LOCAL_ADAPTER, dictAdapter);
        return dictAdapter;
    }

    public JSONObject getObjDictAsync(long[] ids) {
        if (ids.length == 0) {
            return new JSONObject();
        }

        DictObj dictObj = DictObjBuilder.create(DictCodes.DictObj, AppCodes.DictApp);
        DictCondition condition = new DictCondition();
        condition.setLeft(new ColumnExpression("id"));
        condition.setOperation(OperationType.IN_LIST);
        condition.setRight(new ValueListExpression(Arrays.stream(ids).boxed().collect(Collectors.toList())));

        dictObj.addCondition(condition);
        dictObj.search(getDictAdapter());

        if (dictObj.isEmpty()) {
            return new JSONObject();
        }

        JSONObject result = new JSONObject();
        dictObj.getRows().stream().map(this::getDictObjAsync)
                .forEach(obj -> {
                    result.put(obj.getString("id"), obj);
                });
        return result;
    }

    public JSONObject getDictObjAsync(DictObjRow objRow) {
        JSONObject jo = objRow.toJSON();
        jo.put("columns", getUIObjColumns(objRow.get("id").getLongValue()));
        return jo;
    }

    public JSONObject getObjDictAsync(String codes, String appCode) {
        if (StringUtils.isEmpty(codes)) {
            return new JSONObject();
        }

        DictObj dictObj = DictObjBuilder.create(DictCodes.DictObj, AppCodes.DictApp);

        DictCondition condition = new DictCondition();
        condition.setLeft(new ColumnExpression("code"));
        condition.setOperation(OperationType.IN_LIST);
        condition.setRight(new ValueListExpression(Arrays.asList(codes.split(","))));

        dictObj.addCondition(condition);
        dictObj.addCondition("appCode", Optional.ofNullable(appCode).orElse(DictGlobalConfig.Instance.getDefaultApp()));
        dictObj.search(getDictAdapter());

        if (dictObj.isEmpty()) {
            return new JSONObject();
        }

        JSONObject result = new JSONObject();

        dictObj.getRows().stream().map(this::getDictObjAsync)
                .forEach(jsonObject -> {
                    result.put(jsonObject.getString("code"), jsonObject);
                });

        return result;
    }

    private void loadUIDictAsync(UIDict uiDict) {
        DictObj dictUIObj = DictObjBuilder.create(DictCodes.DictUIObjEx, AppCodes.DictApp);
        dictUIObj.addCondition("uiId", uiDict.getId());
        dictUIObj.search(getDictAdapter());
        if (!dictUIObj.isEmpty()) {
            dictUIObj.getRows().stream()
                    .map(r -> r.toMap())
                    .forEach(map -> {
                        uiDict.getObjs().add(ClassUtil.mapToObject(map, UIDict.UIObj.class));
                    });
        }

        DictObj dictAction = DictObjBuilder.create(DictCodes.DictUIAction, AppCodes.DictApp);
        dictAction.addCondition("uiId", uiDict.getId());
        dictAction.search(getDictAdapter());

        if (!dictAction.isEmpty()) {
            dictAction.getRows().stream()
                    .map(r -> r.to(UIDict.UIAction.class))
                    .forEach(uiAction -> uiDict.getActions().add(uiAction));
        }
    }

    private JSONArray getDataDict(JSONArray data, String codes, String categories) {
        if (StringUtils.isNotEmpty(codes)) {
            HashSet<String> codeSets = new HashSet<>(Arrays.asList(codes.split(",")));
            List<Object> list = data.stream().filter(item -> {
                if (!(item instanceof  JSONObject)) { return false; }
                JSONObject jo = (JSONObject) item;
                String code = jo.getString("code");
                return codeSets.contains(code);
            }).collect(Collectors.toList());

            return new JSONArray(list);
        }

        if (StringUtils.isNotEmpty(categories)) {
            HashSet<String> cateSets = new HashSet<>(Arrays.asList(categories.split(",")));
            List<Object> list = data.stream().filter(item -> {
                if (!(item instanceof  JSONObject)) { return false; }
                JSONObject jo = (JSONObject) item;
                String code = jo.getString("category");
                return cateSets.contains(code);
            }).collect(Collectors.toList());

            return new JSONArray(list);
        }

        return  data;
    }

    public JSONArray getDataDict(String codes, String categories) {
        String cacheData = DictCache.getInstance().getCacheValue(DICT_DATA_KEY);
        if (!StringUtils.isEmpty(cacheData)) {
            return  getDataDict(JSONArray.parseArray(cacheData), codes, categories);
        }

        DictObj dictData = new DictObj(DictCodes.DictData, AppCodes.DictApp);
        dictData.addCondition(new DictCondition("isActive", true));
        dictData.search(getDictAdapter());

        JSONArray result = new JSONArray();
        Map<Long, JSONArray> dictMap = new HashMap<>();
        dictData.getRows().forEach(r ->
        {
            JSONObject jo = r.toJSON();
            dictMap.put(r.get("id").getLongValue(), new JSONArray());
            jo.put("items", dictMap.get(r.get("id").getLongValue()));
            result.add(jo);
        });

        DictObj dictDataItem = new DictObj(DictCodes.DictDataItem, AppCodes.DictApp);
        dictDataItem.addCondition(new DictCondition("isActive", true));
        OrderBy orderBy = new OrderBy();
        orderBy.setColumnName("pid");
        dictDataItem.addOrderBy(orderBy);
        dictDataItem.search(getDictAdapter());
        dictDataItem.getRows().forEach(r ->
        {
            long pid = r.get("pId").getLongValue();
            if (dictMap.containsKey(pid)) {
                dictMap.get(pid).add(r.toJSON());
            }
        });

        DictCache.getInstance().setCacheValue(DICT_DATA_KEY, result);
        return getDataDict(result, codes, categories);
    }

    public JSONObject getUIAsync(String menuId) {
        if (StringUtils.isEmpty(menuId)) {
            return new JSONObject();
        }

        DictObj objMenu = DictObjBuilder.create(DictCodes.SysMenu, SysApp);
        objMenu.addCondition(new DictCondition("id", menuId));
        objMenu.search(getDictAdapter());
        if (objMenu.isEmpty()) {
            return new JSONObject();
        }

        DictObjRow menuRow = objMenu.getFirstOrDefaultRow();
        long uiId = menuRow.getLongValue("uiId");
        if (uiId <= 0) {
            return new JSONObject();
        }

        DictObj objUI = new DictObj(DictCodes.DictUI, AppCodes.DictApp);
        objUI.search(uiId, getDictAdapter());
        if (objUI.getRows().size() == 0) {
            return new JSONObject();
        }

        JSONObject ui = menuRow.toJSON();
        ui.put("actions", getActionsAsync(uiId));
        ui.put("objs", getUIObjectsAsync(uiId));
        ui.put("uiId", TypeUtil.toString(uiId));
        ui.put("uiDict", objUI.getFirstOrDefaultRow().toJSON());
        return ui;
    }

    private JSONArray getActionsAsync(long uiId) {
        DictObj actionObj = new DictObj(DictCodes.DictUIAction, AppCodes.DictApp);
        actionObj.addCondition(new DictCondition("uiId", uiId));
        actionObj.search(getDictAdapter());
        JSONArray result = new JSONArray();
        List<DictObjRow> childActions = new ArrayList<>();
        for (DictObjRow actionObjRow : actionObj.getRows()) {
            if (StringUtils.isEmpty(actionObjRow.get("pid").getStringValue())) {
                childActions.add(actionObjRow);
                continue;
            }

            result.add(actionObjRow.toJSON());
        }

        childActions.forEach(r ->
        {
            long pId = r.get("pid").getLongValue();
            Optional<JSONObject> jo = result.stream()
                    .map(j -> {
                        if (j instanceof JSONObject) {
                            return (JSONObject) j;
                        }

                        return null;
                    })
                    .filter(j -> j != null && j.getLongValue("id") == pId)
                    .findFirst();

            if (!jo.isPresent()) {
                return;
            }

            JSONArray children = jo.get().getJSONArray("children");
            children.add(r.toJSON());

            jo.get().put("children", children);
        });

        return result;
    }

    private JSONArray getUIObjectsAsync(long uiId) {
        DictObj dictUIObjEx = new DictObj(DictCodes.DictUIObjEx, AppCodes.DictApp);
        dictUIObjEx.addCondition(new DictCondition("uiId", uiId));

        dictUIObjEx.search(getDictAdapter());
        if (dictUIObjEx.getRows().size() == 0) {
            return new JSONArray();
        }

        List<Object> jObjects = dictUIObjEx.getRows().stream().map(this::getUIObjectAsync).collect(Collectors.toList());
        return new JSONArray(jObjects);
    }

    private JSONObject getUIObjectAsync(DictObjRow objRow) {
        JSONObject jo = objRow.toJSON();
        jo.put("columns", getUIObjColumns(objRow.get("objId").getLongValue()));
        return jo;
    }

    private JSONArray getUIObjColumns(long objId) {
        DictObj uiColumnObj = new DictObj(DictCodes.DictObjColumnEx, AppCodes.DictApp);
        uiColumnObj.addCondition(new DictCondition("objId", objId));
        uiColumnObj.search(getDictAdapter());
        Map<String, JSONObject> userDict = getUserDict(TypeUtil.toString(objId));

        return new JSONArray(uiColumnObj.getRows()
                .stream()
                .filter(r -> !StringUtils.isEmpty(r.get("columnName").getStringValue()))
                .map(r -> {
                    String columnName = r.get("columnName").getStringValue();
                    JSONObject json = getColumnJson(r);
                    if (userDict.containsKey(columnName)) {
                        DictObjUtil.updateUserDict(json, userDict.get(columnName));
                    }

                    return json;
                })
                .collect(Collectors.toList()));
    }

    private JSONObject getColumnJson(DictObjRow row) {
        JSONObject json = row.toJSON();
        if (!json.containsKey("exId")) {
            return json;
        }

        long exId = json.getLongValue("exId");
        if (exId == 0L) {
            json.put("dispIndex", json.get("seqNo"));
            json.put("isSimple", true);
        }

        json.remove("exId");
        return json;
    }

    private Map<String, JSONObject> getUserDict(String objId) {
        try {
            HSqlCommand cmdUserDict = new HSqlCommand("SELECT USER_DICT AS userDict FROM sys_user_dict WHERE OBJ_ID =:id");
            cmdUserDict.setParameter("id", objId);
            IDataAdapter dataAdapter = DictGlobalConfig.Instance.getDataAdapter();
            Map<String, JSONObject> userDict = new HashMap<>();
            if (dataAdapter == null) {
                return  userDict;
            }

            HDataTable dt = dataAdapter.query(cmdUserDict);
            if (dt.isEmpty() || StringUtils.isEmpty(dt.get(0).getString("userDict"))) { return userDict; }
            String json = dt.get(0).getString("userDict");
            JSONArray arr =  JSON.parseArray(json);
            arr.forEach(c -> {
                if (c instanceof JSONObject) {
                    JSONObject jo = (JSONObject) c;
                    userDict.put(jo.getString("columnName"), jo);
                }
            });

            return userDict;

        } catch (Exception ex) {
            LoggerHelper.debug(ex);
        }

        return new HashMap<>();
    }

    public void loadUIObject(DictObj obj) {
        long objId = obj.getId();
        DictObj uiColumnObj = new DictObj(DictCodes.DictObjColumnEx, AppCodes.DictApp);
        uiColumnObj.addCondition(new DictCondition("objId", objId));
        uiColumnObj.search(getDictAdapter());

        Map<String, DictObjRow> nameToUIColumn = new HashMap<>(uiColumnObj.getRows().size());
        uiColumnObj.getRows().forEach(r -> {
            nameToUIColumn.put(r.getStringValue("columnName"), r);
        });

        obj.getColumns().forEach(col -> {
            if (nameToUIColumn.containsKey(col.getColumnName())) {
                DictObjRow uiColumn = nameToUIColumn.get(col.getColumnName());
                String dispName = uiColumn.getStringValue("dispName");
                if (!StringUtils.isEmpty(dispName)) {
                    col.setColumnLabel(dispName);
                }
                col.getExtendedProperties().set("dispPosition", uiColumn.getStringValue("dispPosition"));
                col.getExtendedProperties().set("dispIndex", uiColumn.getStringValue("dispIndex"));
                return;
            }

            col.getExtendedProperties().set("dispPosition", "0");
        });
    }



    public JSONObject getMenuDict(String menuId) {
        String key = MENU_PREFIX + menuId;
        String cacheData = DictCache.getInstance().getCacheValue(key);
        if (!StringUtils.isEmpty(cacheData)) {
            return JSONObject.parseObject(cacheData);
        }

        JSONObject uiDict = getUIAsync(menuId);
        DictCache.getInstance().setCacheValue(key, uiDict.toString());
        return uiDict;
    }
}