package top.daxian1218.dust.edp.ui;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.data.HDataTable;
import top.daxian1218.dust.commons.data.StatType;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.dict.*;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.db.dict.condition.OperationType;
import top.daxian1218.dust.db.dict.condition.expressions.ColumnExpression;
import top.daxian1218.dust.db.dict.condition.expressions.ValueExpression;
import top.daxian1218.dust.db.dict.ref.RefType;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.edp.AppCodes;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.DictCodes;
import top.daxian1218.dust.edp.common.DispPosition;
import top.daxian1218.dust.edp.utils.JsonUtil;
import top.daxian1218.dust.edp.utils.SqlUtil;
import java.util.*;
import static top.daxian1218.dust.db.dict.DictConstant.C_ID;

/**
 * @author daxian
 */
public class MenuStore {
    private String menuId;

    private String title;
    private DictObj master;
    private final List<DictObj> objList = new ArrayList<>();
    private final Map<String, String> state = new HashMap<>();
    private IDataAdapter adapter;
    private UIDict uiDict;
    private IBiz biz;

    public MenuStore(String menuId) {
        this.menuId = menuId;
    }

    public long getUIId() {
        return uiDict != null ? uiDict.getId() : 0;
    }

    public String getMenuId() {
        return menuId;
    }

    public void setMenuId(String menuId) {
        this.menuId = menuId;
    }

    public IBiz getBiz() {
        return biz;
    }

    public void setBiz(IBiz biz) {
        this.biz = biz;
    }

    public UIDict getUiDict() {
        return uiDict;
    }

    public void setUiDict(UIDict uiDict) {
        this.uiDict = uiDict;
    }

    public DictObj getMaster() {
        return master;
    }

    public void setMaster(DictObj master) {
        this.master = master;
    }

    public Map<String, String> getState() {
        return state;
    }

    /**
     * 操作的字典对象，只读集合
     *
     * @return
     */
    public List<DictObj> getObjList() {
        return Collections.unmodifiableList(objList);
    }

    public IDataAdapter getAdapter() {
        return adapter;
    }

    public void setAdapter(IDataAdapter adapter) {
        this.adapter = adapter;
    }

    private void addDictObj(DictObj obj) {
        if (objList.contains(obj)) {
            return;
        }

        objList.add(obj);
    }

    public DictObj getDictObj(String key) {
        for (DictObj obj : this.objList) {
            if (StringUtils.equals(obj.getKey().toUpperCase(), key.toUpperCase())) {
                return obj;
            }

            if (StringUtils.equals(obj.getCode().toUpperCase(), key.toUpperCase())) {
                return obj;
            }

            DictObj detail = findDetail(obj, key);
            if (detail != null) {
                return detail;
            }
        }

        return null;
    }

    private DictObj findDetail(DictObj obj, String key) {
        if (obj.getRelations().size() == 0) {
            return null;
        }

        for (DictObjRelation relation : obj.getRelations()) {
            if (StringUtils.equals(relation.getDetail().getKey().toUpperCase(), key.toUpperCase())) {
                return relation.getDetail();
            }

            if (StringUtils.equals(relation.getDetail().getCode().toUpperCase(), key.toUpperCase())) {
                return relation.getDetail();
            }

            DictObj detail = findDetail(relation.getDetail(), key);
            if (detail != null) {
                return detail;
            }
        }

        return null;
    }

    public boolean containsObj(String key) {
        return this.objList.stream()
                .anyMatch(o -> StringUtils.equals(o.getKey().toUpperCase(), key.toUpperCase()));
    }

    public void loadJSONAsync(JSONArray objs) {
        if (objs == null) {
            return;
        }

        objs.forEach(item ->
        {
            if (!(item instanceof JSONObject)) {
                return;
            }
            JSONObject jo = (JSONObject) item;
            if (!jo.containsKey(C_ID)) {
                return;
            }

            this.addDictObj(loadDictObj(jo));

        });
    }

    public Object doActionAsync(String actionCode) {
        return getBiz().DoActionAsync(adapter, actionCode, this);
    }

    /**
     * 刷新数据
     *
     * @return 数据集
     */
    public List<DictObj> search() {
        return getBiz().SearchAsync(adapter, this);
    }

    public List<DictObj> save() {
        List<DictObj> result = getBiz().Save(adapter, this);
        adapter.commit();
        return result;
    }

    private void generateQueryWhere(DictObj obj) {

        // 界面字典信息
        handleUIQuery(obj);
        Object qp = obj.getQueryParams();
        if (!(qp instanceof JSONObject)) {
            return;
        }

        // 查询条件
        JSONObject queryParams = (JSONObject) qp;

        // 跳转条件
        handleJumpQuery(obj, queryParams.getJSONObject("jumpQuery"));

        // 前端传入的查询条件
        handleSearchQuery(obj, queryParams.getJSONObject("searchQuery"));

        // 关键字段查询
        handleKeywordQuery(obj, queryParams.getJSONObject("keywordQuery"));

        // 高级查询方案
        handleAdvancedQueryAsync(obj, queryParams.getJSONObject("projectQuery"));

        // 参照查询条件
        handleRefQueryAsync(obj, queryParams.getJSONObject("refQuery"));

        // 排序
        handleOrder(obj, queryParams.getString(("order")));
    }

    private void handleOrder(DictObj obj, String order) {
        if (StringUtils.isEmpty(order)) {
            return;
        }

        String[] strs = order.split(",");
        for (String str : strs) {
            String columnName = str.trim();
            String orderType = StringUtils.EMPTY;
            if (StringUtils.isEmpty(columnName)) {
                continue;
            }
            int idx = columnName.indexOf(' ');
            if (idx > 0) {
                orderType = columnName.substring(idx + 1);
                columnName = columnName.substring(0, idx);
            }

            if (!obj.getColumns().contains(columnName)) {
                continue;
            }
            OrderBy orderBy = new OrderBy();
            orderBy.setColumnName(columnName);
            orderBy.setOrderByType(StringUtils.equalsIgnoreCase("desc", orderType) ? OrderByType.DESC : OrderByType.ASC);
            obj.addOrderBy(orderBy);
        }
    }

    private DictObj loadDictObj(JSONObject jo) {
        if (jo == null || jo.isEmpty()) {
            return null;
        }

        DictObj obj = DictObjBuilder.create(jo.getLongValue("id"));
        obj.setKey(Optional.ofNullable(jo.getString("code")).orElse(TypeUtil.toString(obj.getCode())));
        obj.setReadonly(jo.getBooleanValue("readonly"));

        obj.getPageInfo().setPageSize(jo.getIntValue("pageSize"));
        obj.getPageInfo().setStart(jo.getIntValue("pageIndex"));
        obj.getPageInfo().setTotalRows(Optional.ofNullable(jo.getInteger("totalRows")).orElse(-1));

        JSONObject state = jo.getJSONObject("state");
        if (state != null) {
            obj.setState(state);
        }

        //列值，通常用于存储过程，此处对应列默认值
        JSONObject columnParams = Optional.ofNullable(jo.getJSONObject("columnParams")).orElse(new JSONObject());
        JSONObject statParams = Optional.ofNullable(jo.getJSONObject("statParams")).orElse(new JSONObject());
        if (!columnParams.isEmpty() || !statParams.isEmpty()) {
            for (DictObjColumn column : obj.getColumns()) {
                if (columnParams.containsKey(column.getColumnName())) {
                    column.setParaValue(columnParams.getString(column.getColumnName()));
                }

                if (statParams.containsKey(column.getColumnName())) {
                    column.setStatType(StatType.valueOf(statParams.getIntValue(column.getColumnName())));
                }
            }
        }

        obj.setQueryParams(jo.getJSONObject("queryParams"));

        JSONArray jRows = jo.getJSONArray("rows");
        if (jRows != null && jRows.size() > 0) {
            for (Object o : jRows) {
                if (o instanceof JSONObject) {
                    JSONObject jr = (JSONObject) o;
                    JsonUtil.JsonToRow(jr, obj);
                }
            }
        }

        if (jo.containsKey("relations")) {
            JSONArray jsonRelation = jo.getJSONArray("relations");
            jsonRelation.forEach(item -> {
                if (!(item instanceof JSONObject)) {
                    return;
                }

                JSONObject jr = (JSONObject) item;
                DictObj detail = loadDictObj(jr.getJSONObject("detail"));
                if (detail == null) {
                    return;
                }

                String masterKeys = jr.getString("masterKeys");
                String detailKeys = jr.getString("detailKeys");
                if (StringUtils.isEmpty(masterKeys) || StringUtils.isEmpty(detailKeys)) {
                    return;
                }

                obj.bindRelation(detail, masterKeys, detailKeys);
            });
        }

        if (jo.containsKey("headers")) {
            JSONArray headers = jo.getJSONArray("headers");
            handleGridHeader(obj, headers);
        }

        if (this.master == null && Optional.ofNullable(jo.getBoolean("isMaster")).orElse(false)) {
            this.setMaster(obj);
        }

        // 生成对象字典的查询条件
        this.generateQueryWhere(obj);
        return obj;
    }

    private void handleGridHeader(DictObj obj, JSONArray headers) {
        Map<String, Integer> mapHeaders = new HashMap<>(headers.size());
        for(int i = 0; i < headers.size(); i++) {
            JSONObject header = headers.getJSONObject(i);
            if (header != null) {
                mapHeaders.put(header.getString("columnName"), i);
            }
        }

        obj.getColumns().forEach(col -> {
            if (mapHeaders.containsKey(col.getColumnName())) {
                int dispIndex = mapHeaders.get(col.getColumnName());
                JSONObject header = headers.getJSONObject(dispIndex);
                col.getExtendedProperties().set("dispPosition", DispPosition.GRID.getValue());
                col.getExtendedProperties().set("dispIndex", String.valueOf(dispIndex));
                col.setColumnLabel(header.getString("header"));
                return;
            }

            col.getExtendedProperties().set("dispPosition", DispPosition.NONE.getValue());
        });
    }

    private void handleUIQuery(DictObj obj) {
        if (uiDict == null) {
            return;
        }

        Object qp = obj.getQueryParams();
        if (!(qp instanceof JSONObject)) {
            return;
        }

        JSONObject json = (JSONObject) obj.getQueryParams();
        if (json.containsKey("refQuery")) {
            return;
        }

        UIDict.UIObj dict = uiDict.getObjs().stream()
                .filter(uiObj -> StringUtils.equals(uiObj.getCode(), obj.getKey())).findFirst().orElse(null);
        if (dict == null) {
            return;
        }

        if (StringUtils.isNotEmpty(dict.getFixSearch())) {
            String where = SqlUtil.formatSql(dict.getFixSearch(), obj.getCode());
            if (StringUtils.isEmpty(obj.getFixConditionSql())) {
                obj.setFixConditionSql(where);
                return;
            }

            StringBuilder sb = new StringBuilder(obj.getFixConditionSql());
            sb.insert(0, "(");
            sb.append(")");
            sb.append(" AND (");
            sb.append(where);
            sb.append(")");
            obj.setFixConditionSql(sb.toString());
        }
    }

    private void handleJumpQuery(DictObj obj, JSONObject jumpQuery) {
        if (jumpQuery == null || jumpQuery.isEmpty()) {
            return;
        }

        if (jumpQuery.containsKey("todo")) {
            handleTodoQuery(obj, jumpQuery.getLong("todo"));
        }
    }

    private void handleTodoQuery(DictObj obj, Long id) {
        HSqlCommand cmdTodo = new HSqlCommand("select trigger_where from sys_todo where id=:id");
        cmdTodo.setParameter("id", id);
        HDataTable dt = adapter.query(cmdTodo);

        String todoWhere = dt.isEmpty() ? StringUtils.EMPTY : TypeUtil.toString(dt.getScalar());
        if (StringUtils.isNotEmpty(todoWhere)) {
            String where = SqlUtil.formatSql(todoWhere, obj.getCode());
            if (StringUtils.isEmpty(obj.getFixConditionSql())) {
                obj.setFixConditionSql(where);
                return;
            }

            StringBuilder sb = new StringBuilder(obj.getFixConditionSql());
            sb.insert(0, "(");
            sb.append(")");
            sb.append(" AND (");
            sb.append(where);
            sb.append(")");
            obj.setFixConditionSql(sb.toString());
        }
    }

    private void handleAdvancedQueryAsync(DictObj dictObj, JSONObject projectQuery) {
        String menuId = this.menuId;
        if (StringUtils.isEmpty(menuId) || projectQuery == null || projectQuery.isEmpty()) {
            return;
        }

        AdvancedQuery advancedQuery = new AdvancedQuery(this, dictObj, adapter);

        advancedQuery.loadValueAsync(projectQuery);
        dictObj.addFixCondition(advancedQuery.toCondition());
    }

    private void parseCondition(DictObj obj, String field, String value) {

        DictCondition where = DictCondition.parseCondition(obj, field, value);
        if (where == null) {
            if (biz == null) {
                obj.setConditionSql("1=2");
                return;
            }

            DictCondition subWhere = biz.GetUserCondition(adapter, obj, field, value);
            if (subWhere != null) {
                obj.addFixCondition(subWhere);
            }
            return;
        }

        DictObjColumn col = obj.getColumn(field);
        if (col == null) {
            col = obj.getColumn(field.substring(1));
        }

        if (col == null) {
            if (biz == null) {
                obj.setConditionSql("1=2");
                return;
            }

            DictCondition subWhere = biz.GetUserCondition(adapter, obj, field, value);
            if (subWhere != null) {
                obj.addFixCondition(subWhere);
            }
            return;
        }
        obj.addFixCondition(where);
    }

    private void handleSearchQuery(DictObj obj, JSONObject searchQuery) {
        if (searchQuery == null || searchQuery.isEmpty()) {
            return;
        }

        //约束条件
        for (String key : searchQuery.keySet()) {
            String value = searchQuery.getString(key);
            if (StringUtils.isEmpty(value)) {
                continue;
            }

            if (StringUtils.isEmpty(key)) {
                throw new BuzException("field is null");
            }

            parseCondition(obj, key, value);
        }
    }

    private void handleRefQueryAsync(DictObj obj, JSONObject refQuery) {
        if (refQuery == null || refQuery.isEmpty()) {
            return;
        }

        String columnId = refQuery.getString("columnId");
        JSONObject data = refQuery.getJSONObject("D");
        JSONObject masterData = refQuery.getJSONObject("M");
        boolean isQuery = refQuery.getBooleanValue("isQuery");

        if (StringUtils.isEmpty(columnId)) {
            return;
        }

        DictObj refObjColumn = new DictObj(DictCodes.RefObjColumn, AppCodes.DictApp);
        refObjColumn.search(TypeUtil.toLong(columnId));
        if (refObjColumn.getRows().size() == 0) {
            obj.setFixConditionSql("1=2");
            return;
        }

        DictObjRow refColumnInfo = refObjColumn.getRow(0);
        String refField = refColumnInfo.getStringValue("refColumn");
        if (StringUtils.isEmpty(refField)) {
            obj.setFixConditionSql("1=2");
            return;
        }

        if (refObjColumn.getColumn("refType") != null) {
            RefType refType = RefType.valueOf(refColumnInfo.getStringValue("refType"));

            if (refType == RefType.PROP) {
                refObjColumn.addCondition("columnName", refColumnInfo.get("refObj").getStringValue());
                refObjColumn.addCondition("objId", refColumnInfo.get("objId").getLongValue());
                refObjColumn.search();
                if (refObjColumn.getRows().size() == 0) {
                    obj.setFixConditionSql("1=2");
                    return;
                }
                refColumnInfo = refObjColumn.getRow(0);
            }
        }

        if (refQuery.containsKey("value")) {
            String value = refQuery.getString("value");
            String[] cols = refField.split(",");
            boolean strict = refQuery.getBooleanValue("strict");
            DictCondition valueCondition = new DictCondition();
            valueCondition.setLeft(new ColumnExpression(cols[0]));
            valueCondition.setRight(new ValueExpression(value));
            obj.getPollyConditions().add(valueCondition);

            if (!strict) {
                DictCondition likeCondition = new DictCondition();
                for (String col : cols) {
                    DictCondition containValue = new DictCondition();
                    containValue.setLeft(new ColumnExpression(col));
                    containValue.setOperation(OperationType.CONTAIN);
                    containValue.setRight(new ValueExpression(value));
                    containValue.setRequire(false);
                    likeCondition.getSubConditions().add(containValue);
                }

                obj.getPollyConditions().add(likeCondition);
            }
        }

        long objId = refObjColumn.getRow(0).getLongValue("refObj");
        if (obj.getId() != objId) {
            obj.setFixConditionSql("1=2");
            return;
        }

        String refWhere = isQuery ? refColumnInfo.get("refQueryWhere").getStringValue() : refColumnInfo.getStringValue("refEditWhere");
        if (!StringUtils.isEmpty(refWhere)) {
            refWhere = SqlUtil.formatSql(refWhere, obj.getCode(), data, masterData);
            if (StringUtils.isEmpty(obj.getFixConditionSql())) {
                obj.setFixConditionSql(refWhere);
            } else {
                obj.setFixConditionSql(obj.getFixConditionSql() + " AND (" + refWhere + ")");
            }
        }
    }

    private void handleKeywordQuery(DictObj obj, JSONObject keywordQuery) {
        if (keywordQuery == null) {
            return;
        }

        String keyword = keywordQuery.getString("keyword");
        if (StringUtils.isEmpty(keyword)) {
            return;
        }

        if (StringUtils.equals(AdvancedQuery.DEBUG_FLAG, keyword)) {
            obj.addFixCondition(new DictCondition("fakeColumn", AdvancedQuery.DEBUG_FLAG));
            return;
        }

        DictCondition condition = new DictCondition();
        obj.getColumns().forEach(col ->
        {
            if (col.isKeyword()) {
                DictCondition keywordWhere = new DictCondition();
                keywordWhere.setLeft(new ColumnExpression(col.getColumnName()));
                keywordWhere.setOperation(OperationType.CONTAIN);
                keywordWhere.setRight(new ValueExpression(keyword));
                keywordWhere.setRequire(false);

                condition.getSubConditions().add(keywordWhere);
            }
        });
        if (condition.getSubConditions().size() > 0) {
            obj.addFixCondition(condition);
        }
    }


    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }
}

