package top.daxian1218.dust.edp.utils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.data.HDataRow;
import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.DictObjField;
import top.daxian1218.dust.db.dict.DictObjRow;
import top.daxian1218.dust.db.dict.RowState;
import top.daxian1218.dust.edp.AppCodes;
import top.daxian1218.dust.edp.BuzException;

/**
 * @author daxian
 */
public class DictObjUtil {
    public static DictObj getDictObj(String code) {
        return new DictObj(code, AppCodes.DictApp);
    }

    public static DictObj getDataObj(JSONObject inParams, String appCode) {
        String objCode = inParams.getString("objCode");
        if (StringUtils.isEmpty(objCode)) {
            throw new BuzException("objCode is null");
        }
        return new DictObj(appCode, objCode);
    }

    public static DictObj getDataObj(JSONObject inParams) {
        String objCode = inParams.getString("objCode");
        if (StringUtils.isEmpty(objCode)) {
            throw new BuzException("objCode is null");
        }
        return new DictObj(objCode, null);
    }

    public static DictObj getDataObj(String objCode) {
        if (StringUtils.isEmpty(objCode)) {
            throw new BuzException("objCode is null");
        }
        return new DictObj(objCode, null);
    }

    public static void LoadData(JSONObject inParams, DictObj obj) {
        JSONArray updateList = inParams.getJSONArray("updated");
        JSONArray insertList = inParams.getJSONArray("inserted");
        JSONArray deleteList = inParams.getJSONArray("deleted");

        //json to row. case C,U,D
        if (updateList != null && updateList.size() > 0) {
            for (int i = 0; i < updateList.size(); i++) {
                JSONObject jo = updateList.getJSONObject(i);
                jo.put("$rowState", RowState.MODIFIED);
                JsonUtil.JsonToRow(jo, obj);
            }
        }

        if (insertList != null && insertList.size() > 0) {
            for (int i = 0; i < insertList.size(); i++) {
                JSONObject jo = insertList.getJSONObject(i);
                jo.put("$rowState", RowState.ADDED);
                JsonUtil.JsonToRow(jo, obj);
            }
        }

        if (deleteList != null && deleteList.size() > 0) {
            for (int i = 0; i < deleteList.size(); i++) {
                JSONObject jo = deleteList.getJSONObject(i);
                jo.put("$rowState", RowState.DELETED);
                JsonUtil.JsonToRow(jo, obj);
            }
        }
    }

    public static DictObjRow Clone(DictObjRow src, DictObjRow dest) {
        dest.loadData(src.getData());
        return dest;
    }

    public static void updateUserDict(JSONObject col, JSONObject userDict) {
        if (userDict.containsKey("dispIndex")) {
            col.put("dispIndex", userDict.get("dispIndex"));
        }

        if (userDict.containsKey("dispPosition")) {
            col.put("dispPosition", userDict.get("dispPosition"));
        }

        if (userDict.containsKey("dispName")) {
            col.put("dispName", userDict.get("dispName"));
        }

        if (userDict.containsKey("headerWidth")) {
            col.put("headerWidth", userDict.get("headerWidth"));
        }

        if (userDict.containsKey("defaultValue")) {
            col.put("defaultValue", userDict.get("defaultValue"));
        }

        if (userDict.containsKey("isSortable")) {
            col.put("isSortable", userDict.get("isSortable"));
        }

        if (userDict.containsKey("isKeyword")) {
            col.put("isKeyword", userDict.get("isKeyword"));
        }
    }
}