package top.daxian1218.dust.edp.utils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.data.HDataColumn;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.dict.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author daxian
 */
public class JsonUtil {

    public static final String PLACE_HOlDER_START = "*";
    public static final String PREFIX_OLD = "old_";

    private static void isDefaultValue(Object value, DictObjColumn col) {
        String dfValue = col.getDefaultValue();
    }

    public static void JsonToRow(JSONObject json, DictObj obj) {
        RowState rowState = Optional.of(TypeUtil.toEnumByName(RowState.class, json.getString("$rowState"))).orElse(RowState.ADDED);
        DictObjRow newRow = new DictObjRow(obj, rowState != RowState.ADDED);
        newRow.setTag(json.getString("$id"));
        List<DictObjColumn> modifiedColumns = new ArrayList<>();
        obj.getColumns().forEach(col ->
        {
            if (!json.containsKey(col.getColumnName())) {
                return;
            }
            Object value = json.get(col.getColumnName());

            if (rowState == RowState.ADDED && col.isVarValue()
                    && obj.getVariableHandler() != null
                    && obj.getVariableHandler().isHandled(col.getDefaultValue())) {
                return;
            }

            Object oldValue = json.get(PREFIX_OLD + col.getColumnName());
            if (oldValue != null) {
                newRow.setValue(col.getColumnName(), oldValue);
            }
            modifiedColumns.add(col);
        });

        newRow.acceptChanges();
        modifiedColumns.forEach(col -> {
            newRow.setValue(col.getColumnName(), json.get(col.getColumnName()));
        });

        switch (rowState) {
            case MODIFIED:
                obj.modifyRow(newRow);
                break;
            case DELETED:
                obj.deleteRow(newRow);
                break;
            case UNCHANGED:
                obj.addRow(newRow);
                newRow.acceptChanges();
                break;
            default:
                obj.addRow(newRow);
                break;
        }
    }

    public static JSONObject combineJson(JSONObject first, JSONObject second) {
        JSONObject json = new JSONObject();

        if (first != null) {
            json.putAll(first);
        }

        if (second != null) {
            json.putAll(second);
        }

        return json;
    }

    public static JSONObject toSearchObjSchemas(DictObj obj) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("id", TypeUtil.toString(obj.getId()));

        if (!obj.isReadonly()) {
            jsonObject.put("grid", obj.toGridJSON());
            jsonObject.put("columns", getColumnValues(obj));
            jsonObject.put("totalRows", obj.getPageInfo().getTotalRows());
        }

        if (obj.getRelations().size() > 0) {
            List<DictObj> details = obj.getRelations().stream().map(DictObjRelation::getDetail).collect(Collectors.toList());
            jsonObject.put("details", toSearchObjSchemas(details));
        }
        return  jsonObject;
    }

    public static JSONObject getColumnValues(DictObj obj) {
        JSONObject jo = new JSONObject();
        for (DictObjColumn column : obj.getColumns()) {
            JSONObject opt = new JSONObject();
            if (column.getParaValue() != null) {
                opt.put("paraValue", column.getParaValue());
            }

            if (column.getStatValue() != null) {
                opt.put("statValue", column.getStatValue());
            }

            if (!opt.isEmpty()) {
                jo.put(column.getColumnName(), opt);
            }
        }
        return jo;
    }

    public static JSONArray toSearchObjSchemas(List<DictObj> objList) {
        JSONArray arr = new JSONArray();
        objList.forEach(v -> arr.add(toSearchObjSchemas(v)));

        return arr;
    }

    public static JSONArray toSaveObjSchemas(List<DictObj> objList) {
        // 先回刷数据，避免后续AfterSave影响保存的数据
        JSONArray result = new JSONArray();
        objList.forEach(obj -> {
            result.add(toSaveObjSchemas(obj));
        });

        return result;
    }

    public static JSONObject toSaveObjSchemas(DictObj obj) {
        Map<String, JSONObject> updateRows = new HashMap<>(obj.getRows().size());
        obj.getRows().forEach(r ->
        {
            if (StringUtils.isEmpty(r.getTag())) {
                return;
            }

            updateRows.put(r.getTag(), r.toJSON());
        });

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("id", TypeUtil.toString(obj.getId()));
        if (!obj.isReadonly()) {
            jsonObject.put("updateRows", updateRows);
        }

        if (obj.getRelations().size() > 0) {
            List<DictObj> details = obj.getRelations().stream().map(DictObjRelation::getDetail).collect(Collectors.toList());
            jsonObject.put("details", toSaveObjSchemas(details));
        }
        return jsonObject;
    }
}