package top.daxian1218.dust.edp.utils;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.data.HDataTable;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.dict.DataType;
import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.utils.DataUtil;
import top.daxian1218.dust.service.security.SecurityUtils;

import java.util.Optional;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class SqlUtil {

    private static final Pattern PATTERNS = Pattern.compile("\\{([_A-Za-z0-9:]+)}");
    private static final String TABLE_PLACEHOLDER = "T";
    private static final String DATA_PREFIX = "DATA:";
    private static final String SYS_PREFIX = "SYS:";
    private static final String MASTER_PREFIX = "M:";
    public static final String SCRIPT_SPACE = "    ";

    public static String formatSql(String sql, String tableName) {
        return formatSql(sql, tableName, null, null);
    }

    public static String formatSql(String sql, String tableName , JSONObject data, JSONObject masterData) {
        if (StringUtils.isEmpty(tableName)) {
            tableName = "t";
        }

        Matcher m = PATTERNS.matcher(sql);
        while (m.find()) {
            String key = m.group(1);
            String defaultValue = StringUtils.EMPTY;
            if (StringUtils.isEmpty(key)) {
                continue;
            }

            String placeHolder = Pattern.quote(m.group());

            if (StringUtils.equals(key, TABLE_PLACEHOLDER)) {
                sql = sql.replaceFirst(placeHolder, tableName);
                continue;
            }

            if (StringUtils.startsWith(key, DATA_PREFIX)) {
                key = key.substring(5);
                if (key.contains(":")) {
                    String[] strArr =  key.split(":", 2);
                    key = strArr[0];
                    defaultValue = strArr[1];
                }
                String val = data == null ? StringUtils.EMPTY
                        : Optional.ofNullable(data.getString(key)).orElse(defaultValue);
                sql = sql.replaceFirst(placeHolder, val);
            }

            if (StringUtils.startsWith(key, SYS_PREFIX)) {
                key = key.substring(4);
                if (key.contains(":")) {
                    String[] strArr =  key.split(":", 2);
                    key = strArr[0];
                    defaultValue = strArr[1];
                }
                sql = sql.replaceFirst(placeHolder, Optional.ofNullable(SecurityUtils.getUserInfo().get(key)).orElse(defaultValue));
            }

            if (StringUtils.startsWith(key, MASTER_PREFIX)) {
                key = key.substring(2);
                if (key.contains(":")) {
                    String[] strArr =  key.split(":", 2);
                    key = strArr[0];
                    defaultValue = strArr[1];
                }
                String val = masterData == null ? StringUtils.EMPTY
                        : Optional.ofNullable(masterData.getString(key)).orElse(defaultValue);
                sql = sql.replaceFirst(placeHolder, val);
            }
        }

        return sql;
    }


    public static void insertDictData(StringBuilder sqlBuilder, DictObj dictObj) {
        StringBuilder colBuilder = new StringBuilder();
        dictObj.getColumns().forEach(col -> {
            if (colBuilder.length() > 0) {
                colBuilder.append(",");
            }

            colBuilder.append(col.getPhysicalName());
        });

        dictObj.getRows().forEach(r -> {
            StringBuilder valueBuilder = new StringBuilder();
            dictObj.getColumns().forEach(col -> {
                if (valueBuilder.length() > 0) {
                    valueBuilder.append(",");
                }
                valueBuilder.append(toOracleValue(col.getDataType(), r.get(col.getColumnName()).getValue()));
            });
            sqlBuilder.append(SCRIPT_SPACE).append("INSERT INTO ").append(dictObj.getTableName()).append(" (")
                    .append(colBuilder).append(") VALUES(").append(valueBuilder).append(");").append(StringUtils.LF);
        });
    }

    public static void insertDictData(StringBuilder sqlBuilder, HDataTable dt, String tableName) {
        StringBuilder colBuilder = new StringBuilder();
        dt.getColumns().forEach(col -> {
            if (colBuilder.length() > 0) {
                colBuilder.append(",");
            }

            colBuilder.append(col.getName());
        });

        dt.getRows().forEach(r -> {
            StringBuilder valueBuilder = new StringBuilder();
            dt.getColumns().forEach(col -> {
                if (valueBuilder.length() > 0) {
                    valueBuilder.append(",");
                }
                valueBuilder.append(toOracleValue(col.getDataType(), r.getValue(col.getName())));
            });
            sqlBuilder.append(SCRIPT_SPACE).append("INSERT INTO ").append(tableName).append(" (")
                    .append(colBuilder).append(") VALUES(").append(valueBuilder).append(");").append(StringUtils.LF);
        });
    }

    private static String toOracleValue(String type, Object value) {
        return toOracleValue(DataUtil.getDataType(type), value);
    }

    private static String toOracleValue(DataType dataType, Object value) {
        switch (dataType) {
            case CURRENCY:
            case NUMBER:
            case UBIGINT:
            case INT:
                return TypeUtil.toString(value);
            case BOOLEAN:
                return TypeUtil.toBoolean(value) ? "1" : "0";
            case TEXT:
            case STRING:
            case SHORT_DATE:
            case TIME:
                return toSaveOracleString(TypeUtil.toString(value));
            case DATE:
                return "TIMESTAMP '" + TypeUtil.toString(value, "yyyy-MM-dd HH:mm:ss.SSS") + "'";
            default:
                return StringUtils.EMPTY;
        }
    }

    private static String toSaveOracleString(String val) {
        if (!StringUtils.isEmpty(val)) {
            val = val.replace("'", "''");
        }

        return "'" + val + "'";
    }
}
